#ifndef ZNEO2_H
#define ZNEO2_H

////////////////////////////////////////
//
// Free for private and commercial use.
//
// 2008-11-06
//
////////////////////////////////////////

////////////////////////////////////////
// Data Types

#define int8_t		signed char
#define uint8_t		unsigned char

#define int16_t		signed short
#define uint16_t	unsigned short

#define int32_t		signed int
#define uint32_t	unsigned int

#define BOOL		int8_t

////////////////////////////////////////
// Symbolic constants

#define TRUE		1
#define FALSE		0

#define M_SQRT2		1.4142135623730950488016887

#define M_PI		3.141592653589793238462643

#define D2R			(M_PI/180.0)
#define R2D			(180.0/M_PI)

////////////////////////////////////////
// Macros

#if !defined(_BV)
#define _BV(bit)   (1U << (bit))
#endif

// _countof helper
#if !defined(_countof)
#define _countof(_Array) (sizeof(_Array) / sizeof(_Array[0]))
#endif

#define max(a, b)  (((a) > (b)) ? (a) : (b))
#define min(a, b)  (((a) < (b)) ? (a) : (b))

////////////////////////////////////////
// Special Function Registers

// Flags Register (FLAGS)
#define FLAGS_C			0x80	// Carry Flag
#define FLAGS_Z			0x40	// Zero Flag
#define FLAGS_S			0x20	// Sign Flag
#define FLAGS_V			0x10	// Overflow Flag
#define FLAGS_B			0x08	// Blank Flag
#define FLAGS_F1		0x04	// User Flag 1
#define FLAGS_CIRQE		0x02	// Chained Interrupt Enable
#define FLAGS_IRQE		0x01	// Interrupt ENable

// CPU Control Register (CPUCTL)
#define CPUCTL_DMABW			0x03
#define CPUCTL_DMABW_Quarter	0x03	// DMA is allowed one transaction for every three CPU operations
#define CPUCTL_DMABW_Third		0x02	// DMA is allowed one transaction for every two CPU operations
#define CPUCTL_DMABW_Half		0x01	// DMA is allowed one transaction for each CPU operation
#define CPUCTL_DMABW_All		0x00	// DMA can consume 100% of the bus bandwidth

// Trace Control Register (TRACECTL)
#define TRACECTL_TRACEEN	0x80
#define TRACECTL_TRACESEL	0x07

////////////////////////////////////////
// Interrupt

// System Exception Register SYSEXCP  (SYSEXCPH and SYSEXCPL accessed as a word)
#define SYSEXCP_SPOVF	0x8000
#define SYSEXCP_PCOVF	0x4000
#define SYSEXCP_DIV0	0x2000
#define SYSEXCP_DIVOVF	0x1000
#define SYSEXCP_ILL		0x0800
#define SYSEXCP_WDTOSC	0x0004
#define SYSEXCP_PRIOCS	0x0002
#define SYSEXCP_WDT		0x0001

// Fields within LASTIRQ
#define LASTIRQ_IRQADR	0x7C

// IRQx Enable Register (IRQxEN) Priority Level Mask
#define IRQxEN_LEVEL1	0x00FF
#define IRQxEN_LEVEL2	0xFF00
#define IRQxEN_LEVEL3	0xFFFF

// Interrupt Request 0 Register (IRQ0) and Interrupt Request 0 Set Register (IRQ0SET)
#define IRQ0_T2I		0x80
#define IRQ0_T1I		0x40
#define IRQ0_T0I		0x20
#define IRQ0_U0RXI		0x10
#define IRQ0_U0TXI		0x08
#define IRQ0_I2CI		0x04
#define IRQ0_SPII		0x02
#define IRQ0_ADCI		0x01

// IRQ0 Enable Register (IRQ0EN)
#define IRQ0EN_T2EN		0x8080
#define IRQ0EN_T1EN		0x4040
#define IRQ0EN_T0EN		0x2020
#define IRQ0EN_U0RXEN	0x1010
#define IRQ0EN_U0TXEN	0x0808
#define IRQ0EN_I2CEN	0x0404
#define IRQ0EN_SPIEN	0x0202
#define IRQ0EN_ADCEN	0x0101

// Interrupt Request1 Register (IRQ1) and Interrupt Request1 Set register (IRQ1SET)
#define IRQ1_PAD7I		0x80
#define IRQ1_PAD6I		0x40
#define IRQ1_PAD5I		0x20
#define IRQ1_PAD4I		0x10
#define IRQ1_PAD3I		0x08
#define IRQ1_PAD2I		0x04
#define IRQ1_PAD1I		0x02
#define IRQ1_PAD0I		0x01

// IRQ1 Enable Register (IRQ1EN)
#define IRQ1EN_PAD7EN	0x8080
#define IRQ1EN_PAD6EN	0x4040
#define IRQ1EN_PAD5EN	0x2020
#define IRQ1EN_PAD4EN	0x1010
#define IRQ1EN_PAD3EN	0x0808
#define IRQ1EN_PAD2EN	0x0404
#define IRQ1EN_PAD1EN	0x0202
#define IRQ1EN_PAD0EN	0x0101

// Interrupt Request 2 Register (IRQ2) and Interrupt Request 2 Set Register (IRQ2SET)
#define IRQ2_PWMTI		0x80
#define IRQ2_U1RXI		0x40
#define IRQ2_U1TXI		0x20
#define IRQ2_PWMFI		0x10
#define IRQ2_PC3I		0x08
#define IRQ2_DMA3I		0x08
#define IRQ2_PC2I		0x04
#define IRQ2_DMA2I		0x04
#define IRQ2_PC1I		0x02
#define IRQ2_DMA1I		0x02
#define IRQ2_PC0I		0x01
#define IRQ2_DMA0I		0x01

// IRQ2 Enable Register (IRQ2EN)
#define IRQ2EN_PWMTEN	0x8080
#define IRQ2EN_U1RXEN	0x4040
#define IRQ2EN_U1TXEN	0x2020
#define IRQ2EN_PWMFEN	0x1010
#define IRQ2EN_C3EN		0x0808
#define IRQ2EN_DMA3EN	0x0808
#define IRQ2EN_C2EN		0x0404
#define IRQ2EN_DMA2EN	0x0404
#define IRQ2EN_C1EN		0x0202
#define IRQ2EN_DMA1EN	0x0202
#define IRQ2EN_C0EN		0x0101
#define IRQ2EN_DMA0EN	0x0101

////////////////////////////////////////
// Reset

// Reset Status and Control Register (RSTSCR)
#define RSTSCR_POR		0x80
#define RSTSCR_STOP		0x40
#define RSTSCR_WDT		0x20
#define RSTSCR_EXT		0x10
#define RSTSCR_FLT		0x08
#define RSTSCR_USR		0x04
#define RSTSCR_USERRST	0x01

////////////////////////////////////////
// Flash

// Flash Status Register (FSTAT)
#define FSTAT_UNLOCK			0x80
#define FSTAT_FSTAT				0x3F
#define FSTAT_FSTAT_MASS_ERASE	0x20
#define FSTAT_FSTAT_PAGE_ERASE	0x10
#define FSTAT_FSTAT_PROGRAM_OP	0x08
#define FSTAT_FSTAT_IDLE		0x00

// Flash Control Register (FCTL)
#define FCTL_INFO		0x80

// Flash Sector Protect Register (FSECT)
#define FSECT_SECT7		0x80
#define FSECT_SECT6		0x40
#define FSECT_SECT5		0x20
#define FSECT_SECT4		0x10
#define FSECT_SECT3		0x08
#define FSECT_SECT2		0x04
#define FSECT_SECT1		0x02
#define FSECT_SECT0		0x01

// Flash Page Select Register (FPAGE)
#define FPAGE_PAGE		0x01F8

////////////////////////////////////////
// External Interface

#if defined __ZNEO_EXTIF
// External Interface Control Register (EXTCT)
#define EXTCT_BUSSEL			0xC0	
#define EXTCT_BUSSEL_EXT16BIT	0x80
#define EXTCT_BUSSEL_EXT8BIT	0x40
#define EXTCT_BUSSEL_EXT_NO		0x00
#define EXTCT_MEMSIZE			0x30
#define EXTCT_MEMSIZE_INT_NO	0x30
#define EXTCT_MEMSIZE_INT32KB	0x20
#define EXTCT_MEMSIZE_INT64KB	0x10
#define EXTCT_MEMSIZE_INT128KB	0x00

;   Fields within EXTCSx  (EXTCSxH and EXTCSxL accessed as word)
#define EXTCSx_CSxEN			0x8000
#define EXTCSx_POLSEL			0x4000
#define EXTCSx_CSxISA			0x2000
#define EXTCSx_WB				0x0010
#define EXTCSx_PRxWAIT			0x0030
#define EXTCSx_PRxWAIT_3WS		0x0030
#define EXTCSx_PRxWAIT_2WS		0x0020
#define EXTCSx_PRxWAIT_1WS		0x0010
#define EXTCSx_PRxWAIT_0WS		0x0000
#define EXTCSx_CSxWAIT			0x000F
#define EXTCSx_CSxWAIT_15WS		0x000F
#define EXTCSx_CSxWAIT_14WS		0x000E
#define EXTCSx_CSxWAIT_13WS		0x000D
#define EXTCSx_CSxWAIT_12WS		0x000C
#define EXTCSx_CSxWAIT_11WS		0x000B
#define EXTCSx_CSxWAIT_10WS		0x000A
#define EXTCSx_CSxWAIT_9WS		0x0009
#define EXTCSx_CSxWAIT_8WS		0x0008
#define EXTCSx_CSxWAIT_7WS		0x0007
#define EXTCSx_CSxWAIT_6WS		0x0006
#define EXTCSx_CSxWAIT_5WS		0x0005
#define EXTCSx_CSxWAIT_4WS		0x0004
#define EXTCSx_CSxWAIT_3WS		0x0003
#define EXTCSx_CSxWAIT_2WS		0x0002
#define EXTCSx_CSxWAIT_1WS		0x0001
#define EXTCSx_CSxWAIT_0WS		0x0000
#endif __ZNEO_EXTIF

////////////////////////////////////////
// On Chip Debugger

// Line Control Register (DBGLCR)
#define DBGLCR_OE			0x80
#define DBGLCR_TDH			0x40
#define DBGLCR_HDS			0x20
#define DBGLCR_TXFC			0x10
#define DBGLCR_NBEN			0x08
#define DBGLCR_NB			0x04
#define DBGLCR_OUT			0x02
#define DBGLCR_PIN			0x01

// Status Register (DBGSTAT)
#define DBGSTAT_RDRF		0x80
#define DBGSTAT_RXOV		0x40
#define DBGSTAT_RXFE		0x20
#define DBGSTAT_RXBRK		0x10
#define DBGSTAT_TDRE		0x08
#define DBGSTAT_TXCOL		0x04
#define DBGSTAT_RXBUSY		0x02
#define DBGSTAT_TXBUSY		0x01

// Control Register (DBGCTL)
#define DBGCTL_OCDLOCK		0x80
#define DBGCTL_OCDEN		0x40
#define DBGCTL_CRCEN		0x08
#define DBGCTL_UARTEN		0x04
#define DBGCTL_ABCHAR		0x02
#define DBGCTL_ABSRCH		0x01

////////////////////////////////////////
// Oscillator

// Oscillator Control Register (OSCCTL)
#define OSCCTL_UNLOCK1		0xE7
#define OSCCTL_UNLOCK2		0x18

#define OSCCTL_INTEN		0x80
#define OSCCTL_XTLEN		0x40
#define OSCCTL_WDTEN		0x20
#define OSCCTL_POFEN		0x10
#define OSCCTL_WDFEN		0x08
#define OSCCTL_FLPEN		0x04
#define OSCCTL_SCKSEL		0x03
#define OSCCTL_SCKSEL_WDT	0x03
#define OSCCTL_SCKSEL_EXT	0x01
#define OSCCTL_SCKSEL_INT	0x00

////////////////////////////////////////
// ZNEO LIN UART

// LIN-UART Status 0 Register (UxSTAT0)
#define UxSTAT0_RDA			0x80
#define UxSTAT0_PE			0x40	// Normal Mode
#define UxSTAT0_PLE			0x40	// LIN Mode
#define UxSTAT0_OE			0x20
#define UxSTAT0_FE			0x10
#define UxSTAT0_BRKD		0x08
#define UxSTAT0_TDRE		0x04
#define UxSTAT0_TXE			0x02
#define UxSTAT0_CTS			0x01	// Normal Mode
#define UxSTAT0_ATB			0x01	// LIN Mode

// LIN-UART Mode Select and Status Register (UxMDSTAT)
#define UxMDSTAT_MSEL					0xE0
#define UxMDSTAT_MSEL_HWRev				0xE0
#define UxMDSTAT_MSEL_LINProtocol		0x40
#define UxMDSTAT_MSEL_NoiseFilter		0x20
#define UxMDSTAT_MSEL_Normal			0x00

#define UxMDSTAT_STATUS					0x1F
#define UxMDSTAT_STATUS_NE				0x10
#define UxMDSTAT_STATUS_NEWFRM			0x02
#define UxMDSTAT_STATUS_MPRX			0x01
#define UxMDSTAT_STATUS_RxBreakLength	0x0F

// LIN-UART Control 0 Register (UxCTL0)
#define UxCTL0_TEN			0x80
#define UxCTL0_REN			0x40
#define UxCTL0_CTSE			0x20
#define UxCTL0_PEN			0x10
#define UxCTL0_PSEL			0x08
#define UxCTL0_SBRK			0x04
#define UxCTL0_STOP			0x02
#define UxCTL0_LBEN			0x01

// MultiProcessor Control Register (UxCTL1 with MSEL = 000b)
// UART MULTIPROCESSOR mode, IRDA mode, baud rate timer mode...
#define UxCTL1_MPMD						0xA0
#define UxCTL1_MPMD_MatchingDataBytes	0xA0
#define UxCTL1_MPMD_MatchingAddrBytes	0x80
#define UxCTL1_MPMD_AllAddrBytes		0x20
#define UxCTL1_MPMD_AllBytes			0x00
#define UxCTL1_MPEN						0x40
#define UxCTL1_MPBT						0x10
#define UxCTL1_DEPOL					0x08
#define UxCTL1_BRGCTL					0x04
#define UxCTL1_RDAIRQ					0x02
#define UxCTL1_IREN						0x01

// Noise Filter Control Register (UxCTL1 with MSEL = 001b)
// digital noise filter mode
#define UxCTL1_NFEN						0x80
#define UxCTL1_NFCTL					0x70

// LIN Control Register (UxCTL1 with MSEL = 010b)
// LIN mode
#define UxCTL1_LMST						0x80
#define UxCTL1_LSLV						0x40
#define UxCTL1_ABEN						0x20
#define UxCTL1_ABIEN					0x10
#define UxCTL1_LINSTATE					0x0C
#define UxCTL1_LINSTATE_Active			0x0C
#define UxCTL1_LINSTATE_AutoBaud		0x08
#define UxCTL1_LINSTATE_Wait			0x04
#define UxCTL1_LINSTATE_Sleep			0x00
#define UxCTL1_TXBREAKLENGTH			0x03
#define UxCTL1_TXBREAKLENGTH_16BitTimes	0x03
#define UxCTL1_TXBREAKLENGTH_15BitTimes	0x02
#define UxCTL1_TXBREAKLENGTH_14BitTimes	0x01
#define UxCTL1_TXBREAKLENGTH_13BitTimes	0x00

////////////////////////////////////////
// I2C Master/Slave Controller

// I2C Interrupt Status Register (I2CISTAT)
#define I2CISTAT_TDRE		0x80
#define I2CISTAT_RDRF		0x40
#define I2CISTAT_SAM		0x20
#define I2CISTAT_GCA		0x10
#define I2CISTAT_RD			0x08
#define I2CISTAT_ARBLST		0x04
#define I2CISTAT_SPRS		0x02
#define I2CISTAT_NCKI		0x01

// I2C Control Register (I2CCTL)
#define I2CCTL_IEN			0x80
#define I2CCTL_START		0x40
#define I2CCTL_STOP			0x20
#define I2CCTL_BIRQ			0x10
#define I2CCTL_TXI			0x08
#define I2CCTL_NAK			0x04
#define I2CCTL_FLUSH		0x02
#define I2CCTL_FILTEN		0x01

// I2C State Register (I2CSTATE) - Description when DIAG = 0
#define I2CSTATE_ACKV		0x80
#define I2CSTATE_ACK		0x40
#define I2CSTATE_AS			0x20
#define I2CSTATE_DS			0x10
#define I2CSTATE_10B		0x08
#define I2CSTATE_RSTR		0x04
#define I2CSTATE_SCLOUT		0x02
#define I2CSTATE_BUSY		0x01

// I2C Mode Register (I2CMODE)
#define I2CMODE_DMAIF		0x80

#define I2CMODE_MODE		0x60
#define I2CMODE_MODE_11		0x60
#define I2CMODE_MODE_10		0x40
#define I2CMODE_MODE_01		0x20
#define I2CMODE_MODE_00		0x00

#define I2CMODE_IRM			0x10
#define I2CMODE_GCE			0x08

#define I2CMODE_SLA			0x06
#define I2CMODE_SLA_9		0x04
#define I2CMODE_SLA_8		0x02

#define I2CMODE_DIAG		0x01

////////////////////////////////////////
// Enhanced Serial Peripheral Interface (ESPI)

// ESPI Control Register (ESPICTL)
#define ESPICTL_DIRQE				0x80
#define ESPICTL_ESPIEN1				0x40
#define ESPICTL_BRGCTL				0x20
#define ESPICTL_PHASE				0x10
#define ESPICTL_CLKPOL				0x08
#define ESPICTL_WOR					0x04
#define ESPICTL_MMEN				0x02
#define ESPICTL_ESPIEN0				0x01
#define ESPICTL_ESPIEN				0x41
#define ESPICTL_ESPIEN_XmitReceive	0x41
#define ESPICTL_ESPIEN_XmitOnly		0x40
#define ESPICTL_ESPIEN_ReceiveOnly	0x01
#define ESPICTL_ESPIEN_Disabled		0x00

// ESPI Transmit Data Command Register (ESPITDCR)
#define ESPITDCR_TEOF			0x02	// Transmit End of Frame
#define ESPITDCR_SSV			0x01	// Slave Select Value

// ESPI Mode Register (ESPIMODE)
#define ESPIMODE_SSMD			0xE0	// ?ASM? ESPIMODE.SSMD.I2sMode        EQU %60
#define ESPIMODE_SSMD_I2S		0x40	// ?ASM? ESPIMODE.SSMD.SyncFramePulse EQU %40
#define ESPIMODE_SSMD_LOOPBACK	0x20	// ?ASM? ESPIMODE.SSMD.I2SMode        EQU %20
#define ESPIMODE_SSMD_SPI		0x00	// ?ASM? ESPIMODE.SSMD.SPIMode        EQU %00
#define ESPIMODE_NUMBITS		0x1C
#define ESPIMODE_NUMBITS_7		0x1C
#define ESPIMODE_NUMBITS_6		0x18
#define ESPIMODE_NUMBITS_5		0x14
#define ESPIMODE_NUMBITS_4		0x10
#define ESPIMODE_NUMBITS_3		0x0C
#define ESPIMODE_NUMBITS_2		0x08
#define ESPIMODE_NUMBITS_1		0x04
#define ESPIMODE_NUMBITS_8		0x00
#define ESPIMODE_SSIO			0x02
#define ESPIMODE_SSPO			0x01

// ESPI Status Register (ESPISTAT)
#define ESPISTAT_TDRE			0x80	// Transmit Data Register Empty
#define ESPISTAT_TUND			0x40	// Transmit Underrun
#define ESPISTAT_COL			0x20	// Collision
#define ESPISTAT_ABT			0x10	// Slave mode transaction abort
#define ESPISTAT_ROVR			0x08	// Receive Overrun
#define ESPISTAT_RDRF			0x04	// Receive Data Register Full
#define ESPISTAT_TFST			0x02	// Data transfer is currently in progress
#define ESPISTAT_SLAS			0x01	// Slave Select

// ESPI State Register (ESPISTATE)
#define ESPISTATE_SCKI			0x80	// Serial Clock Input
#define ESPISTATE_SDI			0x40	// Serial Data Input
#define ESPISTATE_ESPISTATE		0x3F

// ESPI Tx DMA Descriptor Command Field
#define SPI_DMA_SSV		0x01	// Slave Select Value

// ESPI Tx DMA Descriptor Status field
#define SPI_DMA_COL		0x02
#define SPI_DMA_TUND	0x01

// ESPI Rx DMA Descriptor Status field
#define SPI_DMA_RSS		0x04
#define SPI_DMA_ABT		0x02
#define SPI_DMA_ROVR	0x01

////////////////////////////////////////
// Timer

// Timer 0-2 Control 0+1 Register (TxCTL)
#define TxCTL_TMODE			0x8007
#define TxCTL_TMODE_11		0x8003	// TRIGGERED ONE-SHOT
#define TxCTL_TMODE_10		0x8002	// COMPARATOR COUNTER
#define TxCTL_TMODE_9		0x8001	// CAPTURE RESTART
#define TxCTL_TMODE_8		0x8000	// PWM DUAL OUTPUT
#define TxCTL_TMODE_7		0x0007	// CAPTURE/COMPARE
#define TxCTL_TMODE_6		0x0006	// GATED
#define TxCTL_TMODE_5		0x0005	// COMPARE
#define TxCTL_TMODE_4		0x0004	// CAPTURE
#define TxCTL_TMODE_3		0x0003	// PWM SINGLE OUTPUT
#define TxCTL_TMODE_2		0x0002	// COUNTER
#define TxCTL_TMODE_1		0x0001	// CONTINUOUS
#define TxCTL_TMODE_0		0x0000	// ONE-SHOT

#define TxCTL_TICONFIG		0x6000	// Timer Interrupt Configuration
#define TxCTL_TICONFIG_11	0x6000
#define TxCTL_TICONFIG_10	0x4000
#define TxCTL_TICONFIG_00	0x0000

#define TxCTL_CASCADE		0x1000	// Timer cascade

#define TxCTL_PWMD			0x0E00	// PWM Delay Value
#define TxCTL_PWMD_128_CYC	0x0E00
#define TxCTL_PWMD_64_CYC	0x0C00
#define TxCTL_PWMD_32_CYC	0x0A00
#define TxCTL_PWMD_16_CYC	0x0800
#define TxCTL_PWMD_8_CYC	0x0600
#define TxCTL_PWMD_4_CYC	0x0400
#define TxCTL_PWMD_2_CYC	0x0200
#define TxCTL_PWMD_0_CYC	0x0000

#define TxCTL_INCAP			0x0100	// Input Capture Event
#define TxCTL_TEN			0x0080	// Timer enable
#define TxCTL_TPOL			0x0040	// Timer Input/Output Polarity

#define TxCTL_PRES			0x0038	// Prescaler
#define TxCTL_PRES_DIV128	0x0038
#define TxCTL_PRES_DIV64	0x0030
#define TxCTL_PRES_DIV32	0x0028
#define TxCTL_PRES_DIV16	0x0020
#define TxCTL_PRES_DIV8		0x0018
#define TxCTL_PRES_DIV4		0x0010
#define TxCTL_PRES_DIV2		0x0008
#define TxCTL_PRES_DIV1		0x0000

////////////////////////////////////////
// Multi-Channel PWM Timer

typedef enum
{
	PWM_NA = -1,
	PWM_0H,
	PWM_0L,
	PWM_1H,
	PWM_1L,
	PWM_2H,
	PWM_2L
}
char PWM_CHANNEL;

// PWM Control 0 Register (PWMCTL0)
#define PWMCTL0_OFF_ON		0x80
#define PWMCTL0_OUTCTL		0x40
#define PWMCTL0_ALIGN_C		0x20
#define PWMCTL0_ADCTRIG		0x08
#define PWMCTL0_READY		0x02
#define PWMCTL0_PWMEN		0x01

// PWM Control 1 Register (PWMCTL1)
#define PWMCTL1_RLFREQ		0xC0
#define PWMCTL1_RLFREQ8		0xC0
#define PWMCTL1_RLFREQ4		0x80
#define PWMCTL1_RLFREQ2		0x40
#define PWMCTL1_RLFREQ1		0x00

#define PWMCTL1_INDEN		0x20
#define PWMCTL1_POL2		0x10
#define PWMCTL1_POL1		0x08
#define PWMCTL1_POL0		0x04

#define PWMCTL1_PRES		0x03
#define PWMCTL1_PRES_DIV8	0x03
#define PWMCTL1_PRES_DIV4	0x02
#define PWMCTL1_PRES_DIV2	0x01
#define PWMCTL1_PRES_DIV1	0x00

// PWM Fault Mask Register (PWMFM)
#define PWMFM_DBGMSK		0x20
#define PWMFM_F1MASK		0x04
#define PWMFM_COMASK		0x02
#define PWMFM_FMASK			0x01

// PWM Fault Status Register (PWMFSTAT)
#define PWMFSTAT_RLDFLAG	0x80
#define PWMFSTAT_DBGFLAG	0x20
#define PWMFSTAT_F1FLAG		0x04
#define PWMFSTAT_C0FLAG		0x02
#define PWMFSTAT_FFLAG		0x01

// PWM Fault Control Register (PWMFCTL)
#define PWMFCTL_DBGRST		0x40
#define PWMFCTL_CMP1INT		0x20
#define PWMFCTL_CMP1RST		0x10
#define PWMFCTL_CMPINT		0x08
#define PWMFCTL_CMPRST		0x04
#define PWMFCTL_FAULT0INT	0x02
#define PWMFCTL_FAULT0RST	0x01

// PWM Input Sample Register (PWMIN)
#define PWMIN_FAULT			0x40
#define PWMIN_IN2L			0x20
#define PWMIN_IN2H			0x10
#define PWMIN_IN1L			0x08
#define PWMIN_IN1H			0x04
#define PWMIN_IN0L			0x02
#define PWMIN_IN0H			0x01

// PWM Output Control Register (PWMOUT)
#define PWMOUT_OUT2L		0x20
#define PWMOUT_OUT2H		0x10
#define PWMOUT_OUT1L		0x08
#define PWMOUT_OUT1H		0x04
#define PWMOUT_OUT0L		0x02
#define PWMOUT_OUT0H		0x01

// PWM 0-2 H/L Duty Cycle Register (PWMHxD, PWMLxD; x = 0,1,2)
#define PWMzxD_SIGN			0x8000
#define PWMzxD_DUTY			0x1FFF

// Current-Sense Sample and Hold Control Register (CSSHR0 and CSSHR1)
#define CSSHRx_SHPOL		0x80
#define CSSHRx_HEN			0x40
#define CSSHRx_NHEN			0x20
#define CSSHRx_LEN			0x10
#define CSSHRx_NLEN			0x08
#define CSSHRx_SHPWM2		0x04
#define CSSHRx_SHPWM1		0x02
#define CSSHRx_SHPWM0		0x01

////////////////////////////////////////
// DMA Controller

// DMA Select Register (DMAxREQSEL)
#define DMAxSEL_CHANSTATE			0xF0
#define DMAxSEL_REQSEL				0x0F
#define DMAxSEL_REQSEL_CONTINUOUS	0x00
#define DMAxSEL_REQSEL_TIMER0		0x01
#define DMAxSEL_REQSEL_TIMER1		0x02
#define DMAxSEL_REQSEL_TIMER2		0x03
#define DMAxSEL_REQSEL_UART0RXD		0x04
#define DMAxSEL_REQSEL_UART0TXD		0x05
#define DMAxSEL_REQSEL_UART1RXD		0x06
#define DMAxSEL_REQSEL_UART1TXD		0x07
#define DMAxSEL_REQSEL_I2C_RX		0x08
#define DMAxSEL_REQSEL_I2C_TX		0x09
#define DMAxSEL_REQSEL_SPI_RX		0x0A
#define DMAxSEL_REQSEL_SPI_TX		0x0B
#define DMAxSEL_REQSEL_ADC			0x0C
#define DMAxSEL_REQSEL_DMAxReqPin	0x0F

// DMA Control Register A (DMAxCTL)
#define DMAxCTL_DMAxEN			0x8000
#define DMAxCTL_LOOP			0x4000
#define DMAxCTL_TXSIZE			0x3000
#define DMAxCTL_TXSIZE_QUAD		0x2000
#define DMAxCTL_TXSIZE_WORD		0x1000
#define DMAxCTL_TXSIZE_BYTE		0x0000
#define DMAxCTL_DSTCTL			0x0C00
#define DMAxCTL_DSTCTL_DEC		0x0800
#define DMAxCTL_DSTCTL_INC		0x0400
#define DMAxCTL_DSTCTL_FIX		0x0000
#define DMAxCTL_SRCCTL			0x0300
#define DMAxCTL_SRCCTL_DEC		0x0200
#define DMAxCTL_SRCCTL_INC		0x0100
#define DMAxCTL_SRCCTL_FIX		0x0000
#define DMAxCTL_IEOB			0x0080
#define DMAxCTL_TXFR			0x0040
#define DMAxCTL_EOF				0x0020
#define DMAxCTL_HALT			0x0010
#define DMAxCTL_CMDSTAT			0x000F

typedef struct
{
	uint16_t	m_usControl;
	uint16_t	m_usLength;
	uint32_t	m_uDstAddr;
	uint32_t	m_uSrcAddr;
	uint32_t	m_uListAddr;
}
DMA_LLIST_DESC;	// needs to be aligned on 16 byte boundaries (_Align 16)

////////////////////////////////////////
// Analog Functions

// ADC Control Register (ADCxCTL)
#define ADCxCTL_START		0x80	// ADC0 Start / Busy
#define ADCxCTL_CVTRD		0x40	// Convert On Read
#define ADCxCTL_REFEN		0x20	// Reference Enable
#define ADCxCTL_ADCEN		0x10	// ADC0 Enable
#define ADCxCTL_ANAIN		0x0F	// Analog Input Select

// ADC Data Bits Register ADCxD   (ADCxDH and ADCxDL accessed as a word)
#define ADCxD_ADCD			0xFFC0

// Sample Settling Time (ADCSST)
#define ADCSST_SST			0x1F

// Sample Time (ADCST)
#define ADCST_ST			0x3F

// ADC Clock Prescale Register (ADCCP)
#define ADCCP_DIV16			0x08
#define ADCCP_DIV8			0x04
#define ADCCP_DIV4			0x02
#define ADCCP_DIV2			0x01

// ADC MAX Register (ADCxMAX)
#define ADCxMAX_LASTCHAN	0x0F

// Comparator and Op Amp Control Register (CMPOPC)
#define CMPOPC_OPEN			0x80	// Operational Amplifier Enable
#define CMPOPC_CPISEL		0x10	// Comparator Input Select
#define CMPOPC_CMPIRQ		0x08	// Comparator Interrupt Edge Select
#define CMPOPC_CMPIV		0x04	// PWM Fault Comparator Polarity
#define CMPOPC_CMPOUT		0x02	// Comparator Output Value
#define CMPOPC_CMPEN		0x01	// Comparator Enable

////////////////////////////////////////
// ZNEO Option Trim Registers

// Clock Filter Trim (CLKFLTR)
#define CLKFLTR_DIV4			0x80
#define CLKFLTR_LONGDLY			0x40
#define CLKFLTR_DLY				0x3C
#define CLKFLTR_DLY3			0x20
#define CLKFLTR_DLY2			0x10
#define CLKFLTR_DLY1			0x08
#define CLKFLTR_DLY0			0x04
#define CLKFLTR_FSEL			0x03
#define CLKFLTR_FSEL_Flt3		0x03
#define CLKFLTR_FSEL_Flt2		0x02
#define CLKFLTR_FSEL_Flt1		0x01
#define CLKFLTR_FSEL_None		0x00

// IPO Trim 1 (IPOTRIM1)
#define IPOTRIM1_IPOTEMPTRIM	0xFC
#define IPOTRIM1_IPOTRIM		0x03

// IPO Trim 2 (IPOTRIM2)
#define IPOTRIM2_IPOTRIM		0xFF

// ADC Reference Voltage Trim (ADCTRIM)
#define ADCTRIM_ADCREF			0x1F

////////////////////////////////////////
// General-Purpose Input/Output (GPIO)

// Port A-K Alternate Function Registers (PxAF)
#define PxAF_AF7			0x8080
#define PxAF_AF7_NoAltFunc	0x0000
#define PxAF_AF7_AltFunc1	0x0080
#define PxAF_AF7_AltFunc2	0x8000
#define PxAF_AF7_AltFunc3	0x8080

#define PxAF_AF6			0x4040
#define PxAF_AF6_NoAltFunc	0x0000
#define PxAF_AF6_AltFunc1	0x0040
#define PxAF_AF6_AltFunc2	0x4000
#define PxAF_AF6_AltFunc3	0x4040

#define PxAF_AF5			0x2020
#define PxAF_AF5_NoAltFunc	0x0000
#define PxAF_AF5_AltFunc1	0x0020
#define PxAF_AF5_AltFunc2	0x2000
#define PxAF_AF5_AltFunc3	0x2020

#define PxAF_AF4			0x1010
#define PxAF_AF4_NoAltFunc	0x0000
#define PxAF_AF4_AltFunc1	0x0010
#define PxAF_AF4_AltFunc2	0x1000
#define PxAF_AF4_AltFunc3	0x1010

#define PxAF_AF3			0x0808
#define PxAF_AF3_NoAltFunc	0x0000
#define PxAF_AF3_AltFunc1	0x0008
#define PxAF_AF3_AltFunc2	0x0800
#define PxAF_AF3_AltFunc3	0x0808

#define PxAF_AF2			0x0404
#define PxAF_AF2_NoAltFunc	0x0000
#define PxAF_AF2_AltFunc1	0x0004
#define PxAF_AF2_AltFunc2	0x0400
#define PxAF_AF2_AltFunc3	0x0404

#define PxAF_AF1			0x0202
#define PxAF_AF1_NoAltFunc	0x0000
#define PxAF_AF1_AltFunc1	0x0002
#define PxAF_AF1_AltFunc2	0x0200
#define PxAF_AF1_AltFunc3	0x0202

#define PxAF_AF0			0x0101
#define PxAF_AF0_NoAltFunc	0x0000
#define PxAF_AF0_AltFunc1	0x0001
#define PxAF_AF0_AltFunc2	0x0100
#define PxAF_AF0_AltFunc3	0x0101

// Port A Irq Mux1 Register (PAIMUX1)
#define PAIMUX1_CPIMUX		0x80
#define PAIMUX1_DBGIMUX		0x01

// Port A Irq Mux Register (PAIMUX)
#define PAIMUX_PAIMUX7		0x80
#define PAIMUX_PAIMUX6		0x40
#define PAIMUX_PAIMUX5		0x20
#define PAIMUX_PAIMUX4		0x10
#define PAIMUX_PAIMUX3		0x08
#define PAIMUX_PAIMUX2		0x04
#define PAIMUX_PAIMUX1		0x02
#define PAIMUX_PAIMUX0		0x01

// Port A Irq Edge Register (PAIEDGE)
#define PAIEDGE_PAIEDGE7	0x80
#define PAIEDGE_PAIEDGE6	0x40
#define PAIEDGE_PAIEDGE5	0x20
#define PAIEDGE_PAIEDGE4	0x10
#define PAIEDGE_PAIEDGE3	0x08
#define PAIEDGE_PAIEDGE2	0x04
#define PAIEDGE_PAIEDGE1	0x02
#define PAIEDGE_PAIEDGE0	0x01

// Port C Irq Mux Register (PCIMUX)
#define PCIMUX_PCIMUX3		0x08
#define PCIMUX_PCIMUX2		0x04
#define PCIMUX_PCIMUX1		0x02
#define PCIMUX_PCIMUX0		0x01

// Port pins for SetIOMode()
#define PORTNA	-1
#define PORTA0	(PORTNA + 1)
#define PORTA1	(PORTNA + 2)
#define PORTA2	(PORTNA + 3)
#define PORTA3	(PORTNA + 4)
#define PORTA4	(PORTNA + 5)
#define PORTA5	(PORTNA + 6)
#define PORTA6	(PORTNA + 7)
#define PORTA7	(PORTNA + 8)

#define PORTB0	(PORTNA + 9)
#define PORTB1	(PORTNA + 10)
#define PORTB2	(PORTNA + 11)
#define PORTB3	(PORTNA + 12)
#define PORTB4	(PORTNA + 13)
#define PORTB5	(PORTNA + 14)
#define PORTB6	(PORTNA + 15)
#define PORTB7	(PORTNA + 16)

#define PORTC0	(PORTNA + 17)
#define PORTC1	(PORTNA + 18)
#define PORTC2	(PORTNA + 19)
#define PORTC3	(PORTNA + 20)
#define PORTC4	(PORTNA + 21)
#define PORTC5	(PORTNA + 22)
#define PORTC6	(PORTNA + 23)
#define PORTC7	(PORTNA + 24)

#define PORTD0	(PORTNA + 25)
#define PORTD1	(PORTNA + 26)
#define PORTD2	(PORTNA + 27)
#define PORTD3	(PORTNA + 28)
#define PORTD4	(PORTNA + 29)
#define PORTD5	(PORTNA + 30)
#define PORTD6	(PORTNA + 31)
#define PORTD7	(PORTNA + 32)

#define PORTE0	(PORTNA + 33)
#define PORTE1	(PORTNA + 34)
#define PORTE2	(PORTNA + 35)
#define PORTE3	(PORTNA + 36)
#define PORTE4	(PORTNA + 37)
#define PORTE5	(PORTNA + 38)
#define PORTE6	(PORTNA + 39)
#define PORTE7	(PORTNA + 40)

#define PORTF0	(PORTNA + 41)
#define PORTF1	(PORTNA + 42)
#define PORTF2	(PORTNA + 43)
#define PORTF3	(PORTNA + 44)
#define PORTF4	(PORTNA + 45)
#define PORTF5	(PORTNA + 46)
#define PORTF6	(PORTNA + 47)
#define PORTF7	(PORTNA + 48)

#define PORTG0	(PORTNA + 49)
#define PORTG1	(PORTNA + 50)
#define PORTG2	(PORTNA + 51)
#define PORTG3	(PORTNA + 52)
#define PORTG4	(PORTNA + 53)
#define PORTG5	(PORTNA + 54)
#define PORTG6	(PORTNA + 55)
#define PORTG7	(PORTNA + 56)

#define PORTH0	(PORTNA + 57)
#define PORTH1	(PORTNA + 58)
#define PORTH2	(PORTNA + 59)
#define PORTH3	(PORTNA + 60)
#define PORTH4	(PORTNA + 61)
#define PORTH5	(PORTNA + 62)
#define PORTH6	(PORTNA + 63)
#define PORTH7	(PORTNA + 64)
#define PORT_UPPER	PORTH7

#if defined __ZNEO_PORTJ
#define PORTJ0	(PORTNA + 65)
#define PORTJ1	(PORTNA + 66)
#define PORTJ2	(PORTNA + 67)
#define PORTJ3	(PORTNA + 68)
#define PORTJ4	(PORTNA + 69)
#define PORTJ5	(PORTNA + 70)
#define PORTJ6	(PORTNA + 71)
#define PORTJ7	(PORTNA + 72)
#undef	PORT_UPPER
#define PORT_UPPER	PORTJ7
#endif __ZNEO_PORTJ

#if defined __ZNEO_PORTK
#define PORTK0	(PORTNA + 73)
#define PORTK1	(PORTNA + 74)
#define PORTK2	(PORTNA + 75)
#define PORTK3	(PORTNA + 76)
#define PORTK4	(PORTNA + 77)
#define PORTK5	(PORTNA + 78)
#define PORTK6	(PORTNA + 79)
#define PORTK7	(PORTNA + 80)
#undef	PORT_UPPER
#define PORT_UPPER	PORTK7
#endif __ZNEO_PORTK

// GPIO configuration with SetIOMode()
typedef char GPIO;			// PORTA0 ... PORTK7 for SetIOMode()

#define GPIO_ALT_3	0x03	// Alternate Functions for SetIOMode()
#define GPIO_ALT_2	0x02
#define GPIO_ALT_1	0x01
#define GPIO_ALT_0	0x00

void SetIOMode(GPIO cGPIO, uint8_t ucAltFunc, BOOL bOutput, BOOL bHighCurrent, BOOL bOpenDrain, BOOL bPullUp, BOOL bStopModeRecovery);

// GPIO Input / Output
BOOL PGet(GPIO cGPIO);
void PSet(GPIO cGPIO, BOOL bSet);

// INFORMATION: internal GPIO Pull-Up Resistor round 10.5k

////////////////////////////////////////
// The Delay1ms() functions perform a delay of uTime milliseconds

void Delay1ms(uint32_t uTime);

////////////////////////////////////////
// The _rotl and _rotr functions rotate uValue by iShift bits.
// _rotl rotates the value left. _rotr rotates the value right.
// Both wrap bits rotated off one end of value to the other end.

uint32_t _rotl(uint32_t uValue, int32_t iShift);
uint32_t _rotr(uint32_t uValue, int32_t iShift);

////////////////////////////////////////
// The copysign function returns x but with the sign of y

double copysign(double x, double y);
float copysignf(float x, float y);

////////////////////////////////////////
// The fmax function compares two values and returns the larger one.

double fmax(double x, double y);
float fmaxf(float x, float y);

////////////////////////////////////////
// The fmin function compares two values and returns the smaller one.

double fmin(double x, double y);
float fminf(float x, float y);

////////////////////////////////////////

#endif /* ZNEO2_H */
